@@ -58,6 +58,8 @@ mktime_from_string (char *date_s,
 	struct tm   tm = {0, };
 	char      **fields;
 
+	tm.tm_isdst = -1;
+
 	/* date */
 
 	fields = g_strsplit (date_s, "-", 3);
@@ -131,9 +133,11 @@ process_line (char     *line,
 
 	g_return_if_fail (line != NULL);
 
-	fdata = file_data_new ();
-
 	date_idx = file_list__get_index_from_pattern (line, "%n%n%n%n-%n%n-%n%n %n%n:%n%n");
+	if (date_idx < 0)
+		return;
+
+	fdata = file_data_new ();
 
 	field_size = file_list__get_prev_field (line, date_idx, 1);
 	fdata->size = g_ascii_strtoull (field_size, NULL, 10);
@@ -155,7 +159,7 @@ process_line (char     *line,
 		fields = g_strsplit (field_name, " link to ", 2);
 	}
 
-	name = unescape_str (fields[0]);
+	name = g_strcompress (fields[0]);
 	if (*name == '/') {
 		fdata->full_path = g_strdup (name);
 		fdata->original_path = fdata->full_path;
@@ -172,7 +176,7 @@ process_line (char     *line,
 
 	fdata->dir = line[0] == 'd';
 	if (fdata->dir)
-		fdata->name =  dir_name_from_path (fdata->full_path);
+		fdata->name = dir_name_from_path (fdata->full_path);
 	else
 		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
 	fdata->path = remove_level_from_path (fdata->full_path);
@@ -180,40 +184,44 @@ process_line (char     *line,
 	if (*fdata->name == 0)
 		file_data_free (fdata);
 	else
-		comm->file_list = g_list_prepend (comm->file_list, fdata);
+		fr_command_add_file (comm, fdata);
 }
 
 
 static void
 add_compress_arg (FrCommand *comm)
 {
-	FrCommandTar *comm_tar = FR_COMMAND_TAR (comm);
-
-	switch (comm_tar->compress_prog) {
-	case FR_COMPRESS_PROGRAM_NONE:
-		break;
-
-	case FR_COMPRESS_PROGRAM_GZIP:
+	if (is_mime_type (comm->mime_type, "application/x-compressed-tar"))
 		fr_process_add_arg (comm->process, "-z");
-		break;
 
-	case FR_COMPRESS_PROGRAM_BZIP:
-		fr_process_add_arg (comm->process,
-				    "--use-compress-program=bzip");
-		break;
-
-	case FR_COMPRESS_PROGRAM_BZIP2:
+	else if (is_mime_type (comm->mime_type, "application/x-bzip-compressed-tar"))
 		fr_process_add_arg (comm->process, "--use-compress-program=bzip2");
-		break;
 
-	case FR_COMPRESS_PROGRAM_COMPRESS:
-		fr_process_add_arg (comm->process, "-Z");
-		break;
+	else if (is_mime_type (comm->mime_type, "application/x-tarz")) {
+		if (is_program_in_path ("gzip"))
+			fr_process_add_arg (comm->process, "-z");
+		else
+			fr_process_add_arg (comm->process, "-Z");
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzip-compressed-tar"))
+		fr_process_add_arg (comm->process, "--use-compress-program=lzip");
 
-	case FR_COMPRESS_PROGRAM_LZOP:
-		fr_process_add_arg (comm->process,
-				    "--use-compress-program=lzop");
-		break;
+	else if (is_mime_type (comm->mime_type, "application/x-lzma-compressed-tar"))
+		fr_process_add_arg (comm->process, "--use-compress-program=lzma");
+
+	else if (is_mime_type (comm->mime_type, "application/x-xz-compressed-tar"))
+		fr_process_add_arg (comm->process, "--use-compress-program=xz");
+
+	else if (is_mime_type (comm->mime_type, "application/x-lzop-compressed-tar"))
+		fr_process_add_arg (comm->process, "--use-compress-program=lzop");
+
+	else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+		FrCommandTar *comm_tar = (FrCommandTar*) comm;
+		char         *option;
+
+		option = g_strdup_printf ("--use-compress-program=%s", comm_tar->compress_command);
+		fr_process_add_arg (comm->process, option);
+		g_free (option);
 	}
 }
 
@@ -239,23 +247,28 @@ begin_tar_command (FrCommand *comm)
 
 
 static void
-fr_command_tar_list (FrCommand  *comm,
-		     const char *password)
+fr_command_tar_list (FrCommand *comm)
 {
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
-				      process_line,
-				      comm);
+	fr_process_set_out_line_func (comm->process, process_line, comm);
 
 	begin_tar_command (comm);
 	fr_process_add_arg (comm->process, "--force-local");
+	fr_process_add_arg (comm->process, "--no-wildcards");
 	fr_process_add_arg (comm->process, "-tvf");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	add_compress_arg (comm);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
 
 
+static gboolean
+can_create_a_compressed_archive (FrCommand *comm)
+{
+	return comm->creating_archive && ! is_mime_type (comm->mime_type, "application/x-7z-compressed-tar");
+}
+
+
 static void
 process_line__generic (char     *line,
 		       gpointer  data,
@@ -291,11 +304,11 @@ process_line__add (char     *line,
 
 static void
 fr_command_tar_add (FrCommand     *comm,
+		    const char    *from_file,
 		    GList         *file_list,
 		    const char    *base_dir,
 		    gboolean       update,
-		    const char    *password,
-		    FRCompression  compression)
+		    gboolean       recursive)
 {
 	FrCommandTar *c_tar = FR_COMMAND_TAR (comm);
 	GList        *scan;
@@ -306,26 +319,41 @@ fr_command_tar_add (FrCommand     *comm,
 
 	begin_tar_command (comm);
 	fr_process_add_arg (comm->process, "--force-local");
-	fr_process_add_arg (comm->process, "--no-recursion");
+	if (! recursive)
+		fr_process_add_arg (comm->process, "--no-recursion");
+	fr_process_add_arg (comm->process, "--no-wildcards");
 	fr_process_add_arg (comm->process, "-v");
 	fr_process_add_arg (comm->process, "-p");
 
 	if (base_dir != NULL) {
-		char *e_base_dir = shell_escape (base_dir);
 		fr_process_add_arg (comm->process, "-C");
-		fr_process_add_arg (comm->process, e_base_dir);
-		g_free (e_base_dir);
+		fr_process_add_arg (comm->process, base_dir);
 	}
 
-	if (update)
-		fr_process_add_arg (comm->process, "-uf");
-	else
-		fr_process_add_arg (comm->process, "-rf");
+	if (can_create_a_compressed_archive (comm)) {
+		fr_process_add_arg (comm->process, "-cf");
+		fr_process_add_arg (comm->process, comm->filename);
+		add_compress_arg (comm);
+	}
+	else {
+		if (comm->creating_archive)
+			fr_process_add_arg (comm->process, "-cf");
+		else
+			fr_process_add_arg (comm->process, "-rf");
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+	}
+
+	if (from_file != NULL) {
+		fr_process_add_arg (comm->process, "-T");
+		fr_process_add_arg (comm->process, from_file);
+	}
 
-	fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 	fr_process_add_arg (comm->process, "--");
-	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, scan->data);
+
+	if (from_file == NULL)
+		for (scan = file_list; scan; scan = scan->next)
+			fr_process_add_arg (comm->process, scan->data);
+
 	fr_process_end_command (comm->process);
 }
 
@@ -348,8 +376,9 @@ begin_func__delete (gpointer data)
 
 
 static void
-fr_command_tar_delete (FrCommand *comm,
-		       GList     *file_list)
+fr_command_tar_delete (FrCommand  *comm,
+		       const char *from_file,
+		       GList      *file_list)
 {
 	FrCommandTar *c_tar = FR_COMMAND_TAR (comm);
 	GList        *scan;
@@ -361,14 +390,23 @@ fr_command_tar_delete (FrCommand *comm,
 	begin_tar_command (comm);
 	fr_process_set_begin_func (comm->process, begin_func__delete, comm);
 	fr_process_add_arg (comm->process, "--force-local");
+	fr_process_add_arg (comm->process, "--no-wildcards");
 	fr_process_add_arg (comm->process, "-v");
 	fr_process_add_arg (comm->process, "--delete");
 	fr_process_add_arg (comm->process, "-f");
 	fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 
+	if (from_file != NULL) {
+		fr_process_add_arg (comm->process, "-T");
+		fr_process_add_arg (comm->process, from_file);
+	}
+
 	fr_process_add_arg (comm->process, "--");
-	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, scan->data);
+
+	if (from_file == NULL)
+		for (scan = file_list; scan; scan = scan->next)
+			fr_process_add_arg (comm->process, scan->data);
+
 	fr_process_end_command (comm->process);
 }
 
@@ -383,12 +421,12 @@ process_line__extract (char     *line,
 
 static void
 fr_command_tar_extract (FrCommand  *comm,
+		        const char *from_file,
 			GList      *file_list,
 			const char *dest_dir,
 			gboolean    overwrite,
 			gboolean    skip_older,
-			gboolean    junk_paths,
-			const char *password)
+			gboolean    junk_paths)
 {
 	GList *scan;
 
@@ -398,22 +436,34 @@ fr_command_tar_extract (FrCommand  *comm,
 
 	begin_tar_command (comm);
 	fr_process_add_arg (comm->process, "--force-local");
+	fr_process_add_arg (comm->process, "--no-wildcards");
 	fr_process_add_arg (comm->process, "-v");
 	fr_process_add_arg (comm->process, "-p");
+
+	if (! overwrite)
+		fr_process_add_arg (comm->process, "-k");
+	if (skip_older)
+		fr_process_add_arg (comm->process, "--keep-newer-files");
+
 	fr_process_add_arg (comm->process, "-xf");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	add_compress_arg (comm);
 
 	if (dest_dir != NULL) {
-		char *e_dest_dir = shell_escape (dest_dir);
 		fr_process_add_arg (comm->process, "-C");
-		fr_process_add_arg (comm->process, e_dest_dir);
-		g_free (e_dest_dir);
+		fr_process_add_arg (comm->process, dest_dir);
+	}
+
+	if (from_file != NULL) {
+		fr_process_add_arg (comm->process, "-T");
+		fr_process_add_arg (comm->process, from_file);
 	}
 
 	fr_process_add_arg (comm->process, "--");
-	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, scan->data);
+
+	if (from_file == NULL)
+		for (scan = file_list; scan; scan = scan->next)
+			fr_process_add_arg (comm->process, scan->data);
 
 	fr_process_end_command (comm->process);
 }
@@ -428,22 +478,38 @@ begin_func__recompress (gpointer data)
 }
 
 
+static gboolean
+gzip_continue_func (gpointer user_data)
+{
+	FrCommand *comm = user_data;
+
+	/* ignore gzip warnings */
+
+	if (comm->process->error.status == 2) {
+		comm->process->error.type = FR_PROC_ERROR_NONE;
+		comm->process->error.status = 0;
+		g_clear_error (&comm->process->error.gerror);
+	}
+
+	return comm->process->error.status == 0;
+}
+
+
 static void
-fr_command_tar_recompress (FrCommand     *comm,
-			   FRCompression  compression)
+fr_command_tar_recompress (FrCommand *comm)
 {
 	FrCommandTar *c_tar = FR_COMMAND_TAR (comm);
 	char         *new_name = NULL;
 
-	switch (c_tar->compress_prog) {
-	case FR_COMPRESS_PROGRAM_NONE:
-		break;
+	if (can_create_a_compressed_archive (comm))
+		return;
 
-	case FR_COMPRESS_PROGRAM_GZIP:
+	if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) {
 		fr_process_begin_command (comm->process, "gzip");
 		fr_process_set_sticky (comm->process, TRUE);
 		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
-		switch (compression) {
+		fr_process_set_continue_func (comm->process, gzip_continue_func, comm);
+		switch (comm->compression) {
 		case FR_COMPRESSION_VERY_FAST:
 			fr_process_add_arg (comm->process, "-1"); break;
 		case FR_COMPRESSION_FAST:
@@ -458,34 +524,12 @@ fr_command_tar_recompress (FrCommand     *comm,
 		fr_process_end_command (comm->process);
 
 		new_name = g_strconcat (c_tar->uncomp_filename, ".gz", NULL);
-		break;
-
-	case FR_COMPRESS_PROGRAM_BZIP:
-		fr_process_begin_command (comm->process, "bzip");
-		fr_process_set_sticky (comm->process, TRUE);
-		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
-		switch (compression) {
-		case FR_COMPRESSION_VERY_FAST:
-			fr_process_add_arg (comm->process, "-1"); break;
-		case FR_COMPRESSION_FAST:
-			fr_process_add_arg (comm->process, "-3"); break;
-		case FR_COMPRESSION_NORMAL:
-			fr_process_add_arg (comm->process, "-6"); break;
-		case FR_COMPRESSION_MAXIMUM:
-			fr_process_add_arg (comm->process, "-9"); break;
-		}
-		fr_process_add_arg (comm->process, "-f");
-		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
-		fr_process_end_command (comm->process);
-
-		new_name = g_strconcat (c_tar->uncomp_filename, ".bz", NULL);
-		break;
-
-	case FR_COMPRESS_PROGRAM_BZIP2:
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-bzip-compressed-tar")) {
 		fr_process_begin_command (comm->process, "bzip2");
 		fr_process_set_sticky (comm->process, TRUE);
 		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
-		switch (compression) {
+		switch (comm->compression) {
 		case FR_COMPRESSION_VERY_FAST:
 			fr_process_add_arg (comm->process, "-1"); break;
 		case FR_COMPRESSION_FAST:
@@ -500,9 +544,8 @@ fr_command_tar_recompress (FrCommand     *comm,
 		fr_process_end_command (comm->process);
 
 		new_name = g_strconcat (c_tar->uncomp_filename, ".bz2", NULL);
-		break;
-
-	case FR_COMPRESS_PROGRAM_COMPRESS:
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-tarz")) {
 		fr_process_begin_command (comm->process, "compress");
 		fr_process_set_sticky (comm->process, TRUE);
 		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
@@ -511,13 +554,72 @@ fr_command_tar_recompress (FrCommand     *comm,
 		fr_process_end_command (comm->process);
 
 		new_name = g_strconcat (c_tar->uncomp_filename, ".Z", NULL);
-		break;
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzip-compressed-tar")) {
+		fr_process_begin_command (comm->process, "lzip");
+		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
+		switch (comm->compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-3"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-6"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-9"); break;
+		}
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+		fr_process_end_command (comm->process);
 
-	case FR_COMPRESS_PROGRAM_LZOP:
+		new_name = g_strconcat (c_tar->uncomp_filename, ".lz", NULL);
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzma-compressed-tar")) {
+		fr_process_begin_command (comm->process, "lzma");
+		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
+		switch (comm->compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-3"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-6"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-9"); break;
+		}
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+		fr_process_end_command (comm->process);
+
+		new_name = g_strconcat (c_tar->uncomp_filename, ".lzma", NULL);
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-xz-compressed-tar")) {
+		fr_process_begin_command (comm->process, "xz");
+		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
+		switch (comm->compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-3"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-6"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-9"); break;
+		}
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+		fr_process_end_command (comm->process);
+
+		new_name = g_strconcat (c_tar->uncomp_filename, ".xz", NULL);
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzop-compressed-tar")) {
 		fr_process_begin_command (comm->process, "lzop");
 		fr_process_set_sticky (comm->process, TRUE);
 		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
-		switch (compression) {
+		switch (comm->compression) {
 		case FR_COMPRESSION_VERY_FAST:
 			fr_process_add_arg (comm->process, "-1"); break;
 		case FR_COMPRESSION_FAST:
@@ -533,18 +635,63 @@ fr_command_tar_recompress (FrCommand     *comm,
 		fr_process_end_command (comm->process);
 
 		new_name = g_strconcat (c_tar->uncomp_filename, ".lzo", NULL);
-		break;
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+		FrCommandTar *comm_tar = (FrCommandTar*) comm;
+
+		fr_process_begin_command (comm->process, comm_tar->compress_command);
+		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
+		switch (comm->compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-mx=1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-mx=5"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-mx=5"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-mx=7"); break;
+		}
+		fr_process_add_arg (comm->process, "a");
+		fr_process_add_arg (comm->process, "-bd");
+		fr_process_add_arg (comm->process, "-y");
+		fr_process_add_arg (comm->process, "-l");
+
+		new_name = g_strconcat (c_tar->uncomp_filename, ".7z", NULL);
+		fr_process_add_arg_concat (comm->process, new_name);
+
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+		fr_process_end_command (comm->process);
+
+		/* remove the uncompressed tar */
+
+		fr_process_begin_command (comm->process, "rm");
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+		fr_process_end_command (comm->process);
 	}
 
 	if (c_tar->name_modified) {
+		char *tmp_dir;
+
 		/* Restore original name. */
 
 		fr_process_begin_command (comm->process, "mv");
 		fr_process_set_sticky (comm->process, TRUE);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, new_name);
-		fr_process_add_arg (comm->process, comm->e_filename);
+		fr_process_add_arg (comm->process, comm->filename);
 		fr_process_end_command (comm->process);
+
+		tmp_dir = remove_level_from_path (new_name);
+
+		fr_process_begin_command (comm->process, "rm");
+		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_add_arg (comm->process, "-fr");
+		fr_process_add_arg (comm->process, tmp_dir);
+		fr_process_end_command (comm->process);
+
+		g_free (tmp_dir);
 	}
 
 	g_free (new_name);
@@ -566,62 +713,75 @@ static char *
 get_uncompressed_name (FrCommandTar *c_tar,
 		       const char   *e_filename)
 {
-	char *new_name = g_strdup (e_filename);
-	int   l = strlen (new_name);
+	FrCommand *comm = FR_COMMAND (c_tar);
+	char      *new_name = g_strdup (e_filename);
+	int        l = strlen (new_name);
 
-	switch (c_tar->compress_prog) {
-	case FR_COMPRESS_PROGRAM_NONE:
-		break;
-
-	case FR_COMPRESS_PROGRAM_GZIP:
+	if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) {
 		/* X.tgz     -->  X.tar
 		 * X.tar.gz  -->  X.tar */
 		if (file_extension_is (e_filename, ".tgz")) {
 			new_name[l - 2] = 'a';
 			new_name[l - 1] = 'r';
-		} else if (file_extension_is (e_filename, ".tar.gz"))
+		}
+		else if (file_extension_is (e_filename, ".tar.gz"))
 			new_name[l - 3] = 0;
-		break;
-
-	case FR_COMPRESS_PROGRAM_BZIP:
-		/* X.tbz     -->  X.tar
-		 * X.tar.bz  -->  X.tar */
-		if (file_extension_is (e_filename, ".tbz")) {
-			new_name[l - 2] = 'a';
-			new_name[l - 1] = 'r';
-		} else if (file_extension_is (e_filename, ".tar.bz"))
-			new_name[l - 3] = 0;
-		break;
-
-	case FR_COMPRESS_PROGRAM_BZIP2:
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-bzip-compressed-tar")) {
 		/* X.tbz2    -->  X.tar
 		 * X.tar.bz2 -->  X.tar */
 		if (file_extension_is (e_filename, ".tbz2")) {
 			new_name[l - 3] = 'a';
 			new_name[l - 2] = 'r';
 			new_name[l - 1] = 0;
-		} else if (file_extension_is (e_filename, ".tar.bz2"))
+		}
+		else if (file_extension_is (e_filename, ".tar.bz2"))
 			new_name[l - 4] = 0;
-		break;
-
-	case FR_COMPRESS_PROGRAM_COMPRESS:
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-tarz")) {
 		/* X.taz   -->  X.tar
 		 * X.tar.Z -->  X.tar */
 		if (file_extension_is (e_filename, ".taz"))
 			new_name[l - 1] = 'r';
 		else if (file_extension_is (e_filename, ".tar.Z"))
 			new_name[l - 2] = 0;
-		break;
-
-	case FR_COMPRESS_PROGRAM_LZOP:
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzip-compressed-tar")) {
+		/* X.tlz     -->  X.tar
+		 * X.tar.lz  -->  X.tar */
+		if (file_extension_is (e_filename, ".tlz")) {
+			new_name[l - 2] = 'a';
+			new_name[l - 1] = 'r';
+		}
+		else if (file_extension_is (e_filename, ".tar.lz"))
+			new_name[l - 3] = 0;
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzma-compressed-tar")) {
+		/* X.tar.lzma --> X.tar
+		 * (There doesn't seem to be a shorthand suffix) */
+		if (file_extension_is (e_filename, ".tar.lzma"))
+			new_name[l - 5] = 0;
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-xz-compressed-tar")) {
+		/* X.tar.xz --> X.tar
+		 * (There doesn't seem to be a shorthand suffix) */
+		if (file_extension_is (e_filename, ".tar.xz"))
+			new_name[l - 5] = 0;
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-lzop-compressed-tar")) {
 		/* X.tzo     -->  X.tar
 		 * X.tar.lzo -->  X.tar */
 		if (file_extension_is (e_filename, ".tzo")) {
 			new_name[l - 2] = 'a';
 			new_name[l - 1] = 'r';
-		} else if (file_extension_is (e_filename, ".tar.lzo"))
+		}
+		else if (file_extension_is (e_filename, ".tar.lzo"))
 			new_name[l - 4] = 0;
-		break;
+	}
+	else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+		/* X.tar.7z -->  X.tar */
+		if (file_extension_is (e_filename, ".tar.7z"))
+			new_name[l - 3] = 0;
 	}
 
 	return new_name;
@@ -635,36 +795,17 @@ static char *
 get_temp_name (FrCommandTar *c_tar,
 	       const char   *filepath)
 {
-	char       *dirname = remove_level_from_path (filepath);
-	const char *filename = file_name_from_path (filepath);
-	char       *temp_name = NULL;
-	char       *e_temp_name = NULL;
-	char       *uncomp_temp_name = NULL;
-	static int  count = 0;
-	int         try = 0;
+	char *dirname = remove_level_from_path (filepath);
+	char *template;
+	char *result = NULL;
+	char *temp_name = NULL;
 
-	do {
-		char *tmp_file_name;
-		g_free (temp_name);
-		g_free (uncomp_temp_name);
-		tmp_file_name = g_strdup_printf (".fr.%d.%d.%s",
-						 getpid (),
-						 count++,
-						 filename);
-		temp_name = g_build_filename (dirname, tmp_file_name, NULL);
-		g_free (tmp_file_name);
-		uncomp_temp_name = get_uncompressed_name (c_tar, temp_name);
-	} while ((path_is_file (temp_name)
-		  || path_is_file (uncomp_temp_name))
-		 && (try++ < MAX_TRIES));
+	template = g_strconcat (dirname, "/.fr-XXXXXX", NULL);
+	result = mkdtemp (template);
+	temp_name = g_build_filename (result, file_name_from_path (filepath), NULL);
+	g_free (template);
 
-	g_free (uncomp_temp_name);
-	g_free (dirname);
-
-	e_temp_name = shell_escape (temp_name);
-	g_free (temp_name);
-
-	return e_temp_name;
+	return temp_name;
 }
 
 
@@ -673,53 +814,47 @@ fr_command_tar_uncompress (FrCommand *comm)
 {
 	FrCommandTar *c_tar = FR_COMMAND_TAR (comm);
 	char         *tmp_name;
+	gboolean      archive_exists;
+
+	if (can_create_a_compressed_archive (comm))
+		return;
 
 	if (c_tar->uncomp_filename != NULL) {
 		g_free (c_tar->uncomp_filename);
 		c_tar->uncomp_filename = NULL;
 	}
 
-	c_tar->name_modified = c_tar->compress_prog != FR_COMPRESS_PROGRAM_NONE;
+	{
+		char *uri = g_filename_to_uri (comm->filename, NULL, NULL);
+		archive_exists = uri_exists (uri);
+		g_free (uri);
+	}
+
+	c_tar->name_modified = ! is_mime_type (comm->mime_type, "application/x-tar");
 	if (c_tar->name_modified) {
 		tmp_name = get_temp_name (c_tar, comm->filename);
-		if (path_is_file (comm->filename)) {
+		if (archive_exists) {
 			fr_process_begin_command (comm->process, "mv");
 			fr_process_add_arg (comm->process, "-f");
-			fr_process_add_arg (comm->process, comm->e_filename);
+			fr_process_add_arg (comm->process, comm->filename);
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
-	} else
-		tmp_name = g_strdup (comm->e_filename);
+	}
+	else
+		tmp_name = g_strdup (comm->filename);
 
-	switch (c_tar->compress_prog) {
-	case FR_COMPRESS_PROGRAM_NONE:
-		break;
-
-	case FR_COMPRESS_PROGRAM_GZIP:
-		if (path_is_file (comm->filename)) {
+	if (archive_exists) {
+		if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) {
 			fr_process_begin_command (comm->process, "gzip");
 			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
+			fr_process_set_continue_func (comm->process, gzip_continue_func, comm);
 			fr_process_add_arg (comm->process, "-f");
 			fr_process_add_arg (comm->process, "-d");
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
-		break;
-
-	case FR_COMPRESS_PROGRAM_BZIP:
-		if (path_is_file (comm->filename)) {
-			fr_process_begin_command (comm->process, "bzip");
-			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
-			fr_process_add_arg (comm->process, "-f");
-			fr_process_add_arg (comm->process, "-d");
-			fr_process_add_arg (comm->process, tmp_name);
-			fr_process_end_command (comm->process);
-		}
-		break;
-
-	case FR_COMPRESS_PROGRAM_BZIP2:
-		if (path_is_file (comm->filename)) {
+		else if (is_mime_type (comm->mime_type, "application/x-bzip-compressed-tar")) {
 			fr_process_begin_command (comm->process, "bzip2");
 			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-f");
@@ -727,20 +862,43 @@ fr_command_tar_uncompress (FrCommand *comm)
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
-		break;
-
-	case FR_COMPRESS_PROGRAM_COMPRESS:
-		if (path_is_file (comm->filename)) {
-			fr_process_begin_command (comm->process, "uncompress");
+		else if (is_mime_type (comm->mime_type, "application/x-tarz")) {
+			if (is_program_in_path ("gzip")) {
+				fr_process_begin_command (comm->process, "gzip");
+				fr_process_set_continue_func (comm->process, gzip_continue_func, comm);
+			}
+			else
+				fr_process_begin_command (comm->process, "uncompress");
 			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-f");
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
-		break;
-
-	case FR_COMPRESS_PROGRAM_LZOP:
-		if (path_is_file (comm->filename)) {
+		else if (is_mime_type (comm->mime_type, "application/x-lzip-compressed-tar")) {
+			fr_process_begin_command (comm->process, "lzip");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
+			fr_process_add_arg (comm->process, "-f");
+			fr_process_add_arg (comm->process, "-d");
+			fr_process_add_arg (comm->process, tmp_name);
+			fr_process_end_command (comm->process);
+		}
+		else if (is_mime_type (comm->mime_type, "application/x-lzma-compressed-tar")) {
+			fr_process_begin_command (comm->process, "lzma");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
+			fr_process_add_arg (comm->process, "-f");
+			fr_process_add_arg (comm->process, "-d");
+			fr_process_add_arg (comm->process, tmp_name);
+			fr_process_end_command (comm->process);
+		}
+		else if (is_mime_type (comm->mime_type, "application/x-xz-compressed-tar")) {
+			fr_process_begin_command (comm->process, "xz");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
+			fr_process_add_arg (comm->process, "-f");
+			fr_process_add_arg (comm->process, "-d");
+			fr_process_add_arg (comm->process, tmp_name);
+			fr_process_end_command (comm->process);
+		}
+		else if (is_mime_type (comm->mime_type, "application/x-lzop-compressed-tar")) {
 			fr_process_begin_command (comm->process, "lzop");
 			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-dfU");
@@ -748,7 +906,24 @@ fr_command_tar_uncompress (FrCommand *comm)
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
-		break;
+		else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+			FrCommandTar *comm_tar = (FrCommandTar*) comm;
+
+			fr_process_begin_command (comm->process, comm_tar->compress_command);
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
+			fr_process_add_arg (comm->process, "e");
+			fr_process_add_arg (comm->process, "-bd");
+			fr_process_add_arg (comm->process, "-y");
+			fr_process_add_arg (comm->process, tmp_name);
+			fr_process_end_command (comm->process);
+
+			/* remove the compressed tar */
+
+			fr_process_begin_command (comm->process, "rm");
+			fr_process_add_arg (comm->process, "-f");
+			fr_process_add_arg (comm->process, tmp_name);
+			fr_process_end_command (comm->process);
+		}
 	}
 
 	c_tar->uncomp_filename = get_uncompressed_name (c_tar, tmp_name);
@@ -756,22 +931,116 @@ fr_command_tar_uncompress (FrCommand *comm)
 }
 
 
-static char *
-fr_command_tar_escape (FrCommand     *comm,
-		       const char    *str)
+static void
+fr_command_tar_handle_error (FrCommand   *comm,
+			     FrProcError *error)
 {
-	char *estr;
-	char *estr2;
-	char *estr3;
+	if (error->type != FR_PROC_ERROR_NONE) {
+		if (error->status <= 1)
+			error->type = FR_PROC_ERROR_NONE;
+	}
+}
 
-        estr = escape_str (str, "\\");
-        estr2 = escape_str (estr, "\\$?*'`& !|()@#:;<>");
-        estr3 = escape_str_common (estr2, "[]", '[', ']');
 
-	g_free (estr);
-	g_free (estr2);
+const char *tar_mime_types[] = { "application/x-compressed-tar",
+				 "application/x-bzip-compressed-tar",
+				 "application/x-tar",
+				 "application/x-7z-compressed-tar",
+				 "application/x-lzip-compressed-tar",
+			         "application/x-lzma-compressed-tar",
+			         "application/x-lzop-compressed-tar",
+			         "application/x-tarz",
+				 "application/x-xz-compressed-tar",
+			         NULL };
 
-	return estr3;
+
+const char **
+fr_command_tar_get_mime_types (FrCommand *comm)
+{
+	return tar_mime_types;
+}
+
+
+FrCommandCap
+fr_command_tar_get_capabilities (FrCommand  *comm,
+			         const char *mime_type)
+{
+	FrCommandCap capabilities;
+
+	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
+
+	/* In solaris gtar is present under /usr/sfw/bin */
+	if (! is_program_in_path ("tar") && ! is_program_in_path ("/usr/sfw/bin/gtar"))
+		return capabilities;
+
+	if (is_mime_type (mime_type, "application/x-tar")) {
+		capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
+	else if (is_mime_type (mime_type, "application/x-compressed-tar")) {
+		if (is_program_in_path ("gzip"))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
+	else if (is_mime_type (mime_type, "application/x-bzip-compressed-tar")) {
+		if (is_program_in_path ("bzip2"))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
+	else if (is_mime_type (mime_type, "application/x-tarz")) {
+		if (is_program_in_path ("compress") && is_program_in_path ("uncompress"))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+		else if (is_program_in_path ("gzip"))
+			capabilities |= FR_COMMAND_CAN_READ;
+	}
+	else if (is_mime_type (mime_type, "application/x-lzip-compressed-tar")) {
+		if (is_program_in_path ("lzip"))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
+	else if (is_mime_type (mime_type, "application/x-lzma-compressed-tar")) {
+		if (is_program_in_path ("lzma"))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
+	else if (is_mime_type (mime_type, "application/x-xz-compressed-tar")) {
+		if (is_program_in_path ("xz"))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
+	else if (is_mime_type (mime_type, "application/x-lzop-compressed-tar")) {
+		if (is_program_in_path ("lzop"))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
+	else if (is_mime_type (mime_type, "application/x-7z-compressed-tar")) {
+		char *try_command[3] = { "7za", "7zr", "7z" };
+		int   i;
+
+		for (i = 0; i < G_N_ELEMENTS (try_command); i++) {
+			if (is_program_in_path (try_command[i])) {
+				capabilities |= FR_COMMAND_CAN_WRITE;
+				break;
+			}
+		}
+	}
+
+	return capabilities;
+}
+
+
+static void
+fr_command_tar_set_mime_type (FrCommand  *comm,
+		 	      const char *mime_type)
+{
+	FrCommandTar *comm_tar = FR_COMMAND_TAR (comm);
+
+	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
+
+	if (is_mime_type (mime_type, "application/x-7z-compressed-tar")) {
+		char *try_command[3] = { "7za", "7zr", "7z" };
+		int   i;
+
+		for (i = 0; i < G_N_ELEMENTS (try_command); i++) {
+			if (is_program_in_path (try_command[i])) {
+				comm_tar->compress_command = g_strdup (try_command[i]);
+				break;
+			}
+		}
+	}
 }
 
 
@@ -786,15 +1055,16 @@ fr_command_tar_class_init (FrCommandTarClass *class)
 
 	gobject_class->finalize = fr_command_tar_finalize;
 
-        afc->list         = fr_command_tar_list;
-	afc->add          = fr_command_tar_add;
-	afc->delete       = fr_command_tar_delete;
-	afc->extract      = fr_command_tar_extract;
-
-	afc->recompress   = fr_command_tar_recompress;
-	afc->uncompress   = fr_command_tar_uncompress;
-
-	afc->escape       = fr_command_tar_escape;
+        afc->list             = fr_command_tar_list;
+	afc->add              = fr_command_tar_add;
+	afc->delete           = fr_command_tar_delete;
+	afc->extract          = fr_command_tar_extract;
+	afc->handle_error     = fr_command_tar_handle_error;
+	afc->get_mime_types   = fr_command_tar_get_mime_types;
+	afc->get_capabilities = fr_command_tar_get_capabilities;
+	afc->set_mime_type    = fr_command_tar_set_mime_type;
+	afc->recompress       = fr_command_tar_recompress;
+	afc->uncompress       = fr_command_tar_uncompress;
 }
 
 
@@ -803,19 +1073,20 @@ fr_command_tar_init (FrCommand *comm)
 {
 	FrCommandTar *comm_tar = (FrCommandTar*) comm;
 
-	comm->file_type = FR_FILE_TYPE_TAR;
-
-	comm->propCanModify                = TRUE;
-	comm->propAddCanUpdate             = TRUE;
-	comm->propAddCanReplace            = FALSE;
-	comm->propAddCanStoreFolders       = TRUE;
-	comm->propExtractCanAvoidOverwrite = FALSE;
-	comm->propExtractCanSkipOlder      = FALSE;
-	comm->propExtractCanJunkPaths      = FALSE;
-	comm->propPassword                 = FALSE;
-	comm->propTest                     = FALSE;
+	comm->propAddCanUpdate              = FALSE;
+	comm->propAddCanReplace             = FALSE;
+	comm->propAddCanStoreFolders        = TRUE;
+	comm->propExtractCanAvoidOverwrite  = FALSE;
+	comm->propExtractCanSkipOlder       = TRUE;
+	comm->propExtractCanJunkPaths       = FALSE;
+	comm->propPassword                  = FALSE;
+	comm->propTest                      = FALSE;
+	comm->propCanDeleteNonEmptyFolders  = FALSE;
+	comm->propCanExtractNonEmptyFolders = FALSE;
+	comm->propListFromFile              = TRUE;
 
 	comm_tar->msg = NULL;
+	comm_tar->uncomp_filename = NULL;
 }
 
 
@@ -839,6 +1110,11 @@ fr_command_tar_finalize (GObject *object)
 		comm_tar->msg = NULL;
 	}
 
+	if (comm_tar->compress_command != NULL) {
+		g_free (comm_tar->compress_command);
+		comm_tar->compress_command = NULL;
+	}
+
 	/* Chain up */
         if (G_OBJECT_CLASS (parent_class)->finalize)
 		G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -871,52 +1147,3 @@ fr_command_tar_get_type ()
 
         return type;
 }
-
-
-FrCommand *
-fr_command_tar_new (FrProcess         *process,
-		    const char        *filename,
-		    FRCompressProgram  prog)
-{
-	FrCommand *comm;
-
-	/* In solaris gtar is present under /usr/sfw/bin */
-
-	if ((!is_program_in_path("/usr/sfw/bin/gtar")) &&
-	    (!is_program_in_path("tar"))) {
-		return NULL;
-	}
-
-	if ((prog == FR_COMPRESS_PROGRAM_GZIP) &&
-            (!is_program_in_path("gzip"))) {
-                return NULL;
-        }
-
-        if ((prog == FR_COMPRESS_PROGRAM_BZIP) &&
-            (!is_program_in_path("bzip"))) {
-                return NULL;
-        }
-
-        if ((prog == FR_COMPRESS_PROGRAM_BZIP2) &&
-            (!is_program_in_path("bzip2"))) {
-                return NULL;
-        }
-
-        if ((prog == FR_COMPRESS_PROGRAM_COMPRESS) &&
-            ((!is_program_in_path("compress")) ||
-             (!is_program_in_path("uncompress")))) {
-                return NULL;
-        }
-
-        if ((prog == FR_COMPRESS_PROGRAM_LZOP) &&
-            (!is_program_in_path("lzop"))) {
-                return NULL;
-        }
-
-	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_TAR, NULL));
-	fr_command_construct (comm, process, filename);
-	FR_COMMAND_TAR (comm)->compress_prog = prog;
-	FR_COMMAND_TAR (comm)->uncomp_filename = NULL;
-
-	return comm;
-}
